import yaml
import os
from xdg import BaseDirectory


class ConfigException(Exception):
    pass


config = None


def set(data):
    global config
    try:
        config_dir_path = BaseDirectory.save_config_path("watson-jira")
        path = os.path.join(config_dir_path, "config.yaml")
        stream = open(path, "w")
        yaml.safe_dump(data, stream)
        config = None
    except Exception:
        raise ConfigException("Failed to write config file")


def get():
    global config
    if config is None:
        try:
            config_dir_path = BaseDirectory.load_first_config("watson-jira")
            if config_dir_path == None:
                raise ConfigException("Failed to find config dir")
            path = os.path.join(config_dir_path, "config.yaml")
            stream = open(path)
        except Exception:
            raise ConfigException("Failed to open config file")

        try:
            config = yaml.safe_load(stream)
        except Exception:
            raise ConfigException("Failed to parse config file")

    return config


def mappings():
    config = get()
    if config is None or "mappings" not in config:
        raise ConfigException("Config file must have `mappings` section")
    return config["mappings"]


def jira():
    config = get()
    if config is None or "jira" not in config:
        raise ConfigException("Config file must have `jira` section")

    jiraconfig = config["jira"]
    jira = {
        "server": None,
        "email": None,
        "apiToken": None,
        "pat": None,
        "cookie": None,
    }

    # Server must be specified
    if "server" not in jiraconfig:
        raise ConfigException("JIRA server must be set")

    jira["server"] = jiraconfig["server"]

    # Try to resolve personal access token
    if "personalAccessToken" in jiraconfig:
        jira["pat"] = jiraconfig["personalAccessToken"]
        return jira

    # Try to resolve email and API token
    if "email" in jiraconfig or "apiToken" in jiraconfig:
        if "email" not in jiraconfig or "apiToken" not in jiraconfig:
            raise ConfigException(
                "Auth method with email and API token requires both to be set"
            )

        jira["email"] = jiraconfig["email"]
        jira["apiToken"] = jiraconfig["apiToken"]
        return jira

    # Try to resolve cookie
    if "cookie" in jiraconfig:
        jira["cookie"] = jiraconfig["cookie"]
        return jira

    raise ConfigException("No authentication method configured")


if __name__ == "__main__":
    pass



import json
from datetime import date

import colorama
import click
import simplejson
from colorama import Fore, Style
from dateutil.relativedelta import relativedelta
from dateutil.rrule import DAILY, rrule
from dateutil.parser import parse

from watson_jira_next.src import watson, jira, config

colorama.init(autoreset=True)

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
TODAY = date.today()
TODAY_YMD = TODAY.strftime("%Y-%m-%d")


def to_ymd(datestring):
    return datestring.split("T")[0]


def dates_from(days_ago):
    dates = rrule(DAILY, dtstart=TODAY - relativedelta(days=days_ago), until=TODAY)
    return [d.strftime("%Y-%m-%d") for d in dates]


def sync_logs(logs):
    if not logs:
        print("No logs")
    else:
        print(Fore.YELLOW + "\nSyncing to JIRA")

    for log in logs:
        started_datetime = parse(log["started"])
        started_formatted = started_datetime.strftime("%H:%M")
        print(
            f"{Fore.BLUE}{log['issue']}{Fore.RESET} at {Fore.GREEN}{started_formatted}{Fore.RESET} {log['timeSpent']}m ",
            end="",
        )

        worklogs = jira.get_worklogs(log["issue"])
        if any(
            [
                log["comment"] == wl["comment"]
                and started_datetime.date() == parse(wl["started"]).date()
                for wl in worklogs
            ]
        ):
            print(Fore.YELLOW + "already exists")
        else:
            jira.add_worklog(**log)
            print(Fore.GREEN + "synced")

    return True


def jira_connect():
    try:
        connection_info = jira.connect()
        if connection_info:
            (server, auth_method) = connection_info
            print(f"Connecting to {server}")
            if auth_method == "pat":
                print("Using personal access token auth method")
            elif auth_method == "apiToken":
                print("Using email with API token auth method")
            else:
                print("Using cookie auth method")

            return True
    except config.ConfigException as e:
        click.echo(Fore.RED + f"Configuration error: {e}")
        click.echo(Fore.LIGHTBLACK_EX + "You can try to run 'watson-jira init'")
    except jira.JiraException as e:
        click.echo(Fore.RED + f"JIRA error: {e}")

    return False


def check_connection():
    try:
        jira.connect()
        current_user = jira.test_conn()
        if current_user:
            click.echo(Fore.GREEN + f"Connected as {current_user}")
            click.echo(
                Fore.LIGHTBLACK_EX
                + f"Please make sure to define mappings in the config file (default in ~/.config/watson-jira/)"
            )
            return True
    except Exception:
        pass
    return False


@click.group(context_settings=CONTEXT_SETTINGS)
@click.version_option(version="1.0.0")
def main():
    pass


@main.command()
@click.option("--date", default=None, help="date to sync logs")
@click.option("--from", default=0, type=int, help="sync logs from this long ago")
@click.option("--issue", default=None, help="only sync logs for this issue")
@click.option(
    "--interactive",
    is_flag=True,
    help="enable propmts to confirm or change target issue",
)
def sync(**kwargs):
    if jira_connect():
        days_ago = kwargs["from"]
        date = kwargs["date"]
        issue = kwargs["issue"]
        is_interactive = kwargs["interactive"]

        datelist = dates_from(days_ago)
        if date:  # specific date trumps a range
            datelist = [date]

        for date in datelist:
            print()
            print(Fore.CYAN + Style.NORMAL + f"{date}")
            logs = watson.log_day(
                date, tempo_format=True, is_interactive=is_interactive
            )
            if issue:
                logs = [l for l in logs if l["issue"] == issue]

            sync_logs(logs)

        print(Fore.CYAN + "\nSynchronization finished\n" + Fore.RESET)


@main.command()
@click.option(
    "--issue", default=None, required=True, help="get worklogs from this issue"
)
@click.option("--id", default=None, help="get specific worklog by id")
def tempo(**kwargs):
    if jira_connect():
        issue = kwargs["issue"]
        _id = kwargs["id"]
        if _id:
            worklogs = jira.get_worklog(issue, _id)
        else:
            worklogs = jira.get_worklogs(issue)
        click.echo(simplejson.dumps(worklogs, skipkeys=True))


@main.command()
@click.option("--date", default=TODAY_YMD, help="date to get logs")
@click.option("--tempo-format", is_flag=True, help="format logs for tempo timesheet")
def logs(**kwargs):
    logs = watson.log_day(kwargs["date"], kwargs["tempo_format"])
    click.echo(json.dumps(logs))


@main.command()
@click.option("--clean-existing", is_flag=True, help="override existing config")
def init(**kwargs):
    if not kwargs["clean_existing"] and check_connection():
        return

    data = {}
    data["jira"] = {}
    data["jira"]["server"] = click.prompt(Fore.BLUE + "Jira server URL", type=str)

    auth_method = click.prompt(
        f"""{Fore.BLUE}Jira authentication method
  {Fore.RESET}0 {Fore.LIGHTBLACK_EX}={Fore.BLUE} Personal access token
  {Fore.RESET}1 {Fore.LIGHTBLACK_EX}={Fore.BLUE} Email and Api token
  {Fore.RESET}2 {Fore.LIGHTBLACK_EX}={Fore.BLUE} Cookie
Your selection{Fore.RESET}""",
        type=int,
        default="0",
    )

    if auth_method == 0:
        data["jira"]["personalAccessToken"] = click.prompt(
            Fore.BLUE + "Personal access token", type=str
        )
    elif auth_method == 1:
        data["jira"]["email"] = click.prompt(Fore.BLUE + "Jira email", type=str)
        click.echo(
            Fore.LIGHTBLACK_EX
            + "Create token at https://id.atlassian.com/manage/api-tokens#"
        )
        data["jira"]["apiToken"] = click.prompt(Fore.BLUE + "Api token", type=str)
    elif auth_method == 2:
        click.echo(
            Fore.LIGHTBLACK_EX
            + "In browser open developer tools and Network tab.\nIf no request is visible, then refresh page.\nOpen details of any GET request, and copy 'Cookie' from the Request Headers section.\nIt's ok to paste also with 'Cookie: ' field name."
        )
        cookie = click.prompt(Fore.BLUE + "Cookie", type=str)
        if cookie.startswith("Cookie: "):
            cookie = cookie[cookie.find(" ") + 1 :]
        data["jira"]["cookie"] = cookie
    else:
        click.echo(Fore.RED + f"Invalid value")
        return

    data["mappings"] = []

    config.set(data)
    jira.invalidate()

    if not check_connection():
        click.echo(
            Fore.RED
            + "Unable to fetch user's display name with provided configuration!"
        )


if __name__ == "__main__":
    main()



from dateutil.parser import parse
from watson_jira_next.src import config
from jira import JIRA
from typing import Optional

jira: Optional[JIRA] = None


class JiraException(Exception):
    pass


def invalidate():
    global jira
    jira = None


def connect():
    global jira
    if jira is None:
        jiraconfig = config.jira()
        try:
            if jiraconfig["pat"]:
                auth_method = "pat"
                headers = JIRA.DEFAULT_OPTIONS["headers"].copy()
                headers["Authorization"] = jiraconfig["pat"]
                jira = JIRA(server=jiraconfig["server"], options={"headers": headers})

            elif jiraconfig["apiToken"]:
                auth_method = "apiToken"
                auth = (jiraconfig["email"], jiraconfig["apiToken"])
                jira = JIRA(server=jiraconfig["server"], basic_auth=auth)

            else:
                auth_method = "cookie"
                headers = JIRA.DEFAULT_OPTIONS["headers"].copy()
                headers["cookie"] = jiraconfig["cookie"]
                jira = JIRA(server=jiraconfig["server"], options={"headers": headers})

            return (jiraconfig["server"], auth_method)
        except Exception:
            raise JiraException("Connection failed")


def get_worklog(issue, _id):
    global jira
    if jira is None:
        raise JiraException("Jira not connected")
    worklog = jira.worklog(issue, _id)
    return vars(worklog)


def get_worklogs(issue):
    global jira
    if jira is None:
        raise JiraException("Jira not connected")
    worklogs = jira.worklogs(issue)
    parsed_worklogs = []
    for worklog in worklogs:
        wl = {
            "issue": issue,
            "comment": worklog.comment if hasattr(worklog, "comment") else None,
            "started": worklog.started,
            "timeSpent": worklog.timeSpent,
            "id": worklog.id,
        }
        parsed_worklogs.append(wl)
    return parsed_worklogs


def add_worklog(issue, timeSpent, comment, started):
    global jira
    if jira is None:
        raise JiraException("Jira not connected")
    wl = jira.add_worklog(
        issue, timeSpent=timeSpent, comment=comment, started=parse(started)
    )
    return wl


def test_conn():
    global jira
    if jira is None:
        raise JiraException("Jira not connected")
    user = jira.current_user("displayName")
    try:
        assert user != "anonymous"
    except AssertionError:
        return False
    return user


if __name__ == "__main__":
    pass



from setuptools import setup


def readme():
    with open("README.md") as f:
        return f.read()


setup(
    name="watson_jira_next",
    version="0.3.2",
    description="Format and upload Watson time logs to Jira as Tempo worklogs",
    long_description=readme(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Topic :: Office/Business :: Scheduling",
    ],
    url="https://github.com/PrimaMateria/watson-jira-next",
    author="Jonathan Medwig, Matus Benko",
    author_email="jonmedwig@gmail.com, matus.benko@gmail.com",
    license="MIT",
    packages=["watson_jira_next", "watson_jira_next.src"],
    install_requires=[
        "td-watson",
        "python-dateutil",
        "click >=7.0,<8.0",
        "simplejson",
        "colorama",
        "jira",
        "pyyaml",
        "pyxdg"
    ],
    setup_requires=["pytest-runner"],
    tests_require=["pytest"],
    entry_points={"console_scripts": ["watson-jira=watson_jira_next.cli:main"]},
    zip_safe=False,
    include_package_data=True,
)



import json
from subprocess import Popen, PIPE
from datetime import datetime
from colorama import Fore

from watson_jira_next.src import mapper


def logs_to_worklogs(logs, is_interactive):
    """Convert Watson logs to Tempo (Jira) worklog dictionaries"""
    print(Fore.YELLOW + "Mapping watson logs to JIRA tickets")
    worklogs = []
    for log in logs:
        jira_issue = mapper.map(log["project"], log["tags"], is_interactive)
        if jira_issue is None:
            continue

        worklog = {
            "started": log["start"],
            "issue": jira_issue,
            "comment": get_comment(log["id"], log["project"], log["tags"]),
            "timeSpent": get_time_spent(log["start"], log["stop"]),
        }
        worklogs.append(worklog)
    return worklogs


def get_comment(id, project, tags):
    return "{0}\n{1} - [{2}]".format(id, project, ", ".join(tags))


def get_time_spent(start, stop):
    datetime_start = datetime.fromisoformat(start)
    datetime_stop = datetime.fromisoformat(stop)
    return int((datetime_stop - datetime_start).total_seconds() // 60 or 1)


def log_day(date, tempo_format=False, is_interactive=False):
    """Get Watson logs for given date in JSON"""
    process = Popen(
        ["watson", "log", "--from", date, "--to", date, "--json"],
        stdout=PIPE,
        stderr=PIPE,
    )
    stdout, _ = process.communicate()
    logs = json.loads(stdout.decode("ascii").strip())
    if tempo_format:
        logs = logs_to_worklogs(logs, is_interactive)
    return logs



import re
import click
from colorama import Fore

from watson_jira_next.src import config


def is_jira_issue(string):
    """Returns True if input string is a valid JIRA issue key, else False"""
    jira_regex = r"^[A-Z]{1,10}-[0-9]+$"
    return bool(re.match(jira_regex, string))


def process_single_issue(mapping):
    return mapping["issue"]


def process_issue_per_project(mapping, project):
    if project in mapping["projects"].keys():
        return mapping["projects"][project]
    return None


def process_issue_specified_in_tag(tags):
    for tag in tags:
        if is_jira_issue(tag):
            return tag
    return None


def ask():
    return click.prompt("Specify jira issue (leave empty to skip)", default="")


def map(project, tags, is_interactive):
    mappings = config.mappings()

    # resolve jira issue from the tag
    jira_issue = None
    for mapping in mappings:
        if mapping["name"] in tags:
            if mapping["type"] == "single_issue":
                jira_issue = process_single_issue(mapping)
            elif mapping["type"] == "issue_per_project":
                jira_issue = process_issue_per_project(mapping, project)
            elif mapping["type"] == "issue_specified_in_tag":
                jira_issue = process_issue_specified_in_tag(tags)

    # backward compatibility - resolve jira issue from project name
    if jira_issue is None and is_jira_issue(project):
        jira_issue = project

    # print the status
    styled_log = get_styled_log(project, tags)
    if jira_issue is None:
        click.echo(f"{styled_log} {Fore.RED}unresolved{Fore.RESET}")
    else:
        click.echo(f"{styled_log} {Fore.GREEN}{jira_issue}{Fore.RESET}")

    # interact with user
    if is_interactive and jira_issue is None:
        jira_issue = ask()

    if not jira_issue:
        jira_issue = None
    return jira_issue


def get_styled_log(project, tags):
    out = Fore.MAGENTA + f"{project}"
    tag_delimeter = f"{Fore.RESET},{Fore.BLUE} "
    if len(tags):
        out += f"  {Fore.RESET}[{Fore.BLUE}{tag_delimeter.join(tags)}{Fore.RESET}]"
    return out


if __name__ == "__main__":
    pass
